@@ -16,33 +16,38 @@
 -include_lib("ctool/include/test/test_utils.hrl").
 -include_lib("ctool/include/test/assertions.hrl").
 -include_lib("ctool/include/test/performance.hrl").
+-include_lib("ctool/include/oz/oz_providers.hrl").
+
 
 %% API
--export([all/0, init_per_testcase/2, end_per_testcase/2, init_per_suite/1]).
+-export([all/0, init_per_suite/1, end_per_suite/1, init_per_testcase/2, end_per_testcase/2]).
+%%-export([all/0, init_per_testcase/2, end_per_testcase/2, init_per_suite/1, end_per_suite/1]).
 
 -export([less_than_block_fetch_test/1, exact_block_size_fetch_test/1,
     more_than_block_fetch_test/1, more_than_block_fetch_test2/1, cancel_fetch_test/1,
     error_open_fun_test/1, error_read_fun_test/1, error_write_fun_test/1, many_requests_test/1,
     many_same_requests_test/1, many_requests_to_one_file/1, request_bigger_than_file_test/1,
-    offset_greater_than_file_size_test/1]).
+    offset_greater_than_file_size_test/1, restart_gateway_test/1, restart_rtransfer_server_test/1]).
 
 -export([read_fun/1, write_fun/1, counter/1, onCompleteCounter/1, data_counter/2]).
 
 all() ->
     ?ALL([
         less_than_block_fetch_test,
+        restart_gateway_test,
+        restart_rtransfer_server_test,
         exact_block_size_fetch_test,
         more_than_block_fetch_test,
         more_than_block_fetch_test2,
-        %% TODO - uncomment below test after resolving VFS-1573
-        %% cancel_fetch_test,
+%%         TODO - uncomment below test after resolving VFS-1573
+%%         cancel_fetch_test,
         many_requests_test,
         many_same_requests_test,
         many_requests_to_one_file,
-        %% TODO - uncomment below 3 tests after resolving VFS-1574
-        %% error_open_fun_test,
-        %% error_read_fun_test,
-        %% error_write_fun_test,
+%%         TODO - uncomment below 3 tests after resolving VFS-1574
+%%         error_open_fun_test,
+%%         error_read_fun_test,
+%%         error_write_fun_test,
         offset_greater_than_file_size_test,
         request_bigger_than_file_test
     ]).
@@ -56,6 +61,7 @@ all() ->
 -define(TEST_OFFSET, 0).
 -define(TIMEOUT, timer:seconds(120)).
 -define(SLEEP_TIMEOUT, timer:seconds(120)).
+-define(STREAMS_NUM, 50).
 
 -define(DEFAULT_RTRANSFER_OPTS,
     [
@@ -69,7 +75,8 @@ all() ->
                 {transport, ranch_tcp},
                 {trans_opts, [{port, ?RTRANSFER_PORT}]}
             ]
-        }
+        },
+        {bind, lists:duplicate(?STREAMS_NUM, {0, 0, 0, 0})}
     ]
 ).
 
@@ -89,8 +96,8 @@ less_than_block_fetch_test(Config) ->
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     CounterPid = spawn(?MODULE, counter, [0]),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -103,6 +110,60 @@ less_than_block_fetch_test(Config) ->
     stop_counter(CounterPid),
     ?assertReceivedMatch({counter, 1}, ?TIMEOUT).
 
+restart_gateway_test(Config) ->
+    %% test fetching data of size smaller then rtransfer block_size
+    [Worker1, Worker2 | _] = ?config(op_worker_nodes, Config),
+
+    %% given
+    DataSize = 32,
+    Data = generate_binary(DataSize),
+    ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
+    WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
+    CounterPid = spawn(?MODULE, counter, [0]),
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+
+    %% when
+    ?assertMatch({ok, _}, start_gateway(Worker1, RtransferOpts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, RtransferOpts2)),
+
+    Ref = prepare_fetch_request(Worker1, Worker2, ?TEST_FILE_UUID, ?TEST_OFFSET, DataSize),
+    ok = rpc:call(Worker1, gen_server2, stop, [gateway, test_reason, infinity]),
+    timer:sleep(timer:seconds(1)),
+    fetch_data(Worker1, Ref, notify_fun(CounterPid), on_complete_fun(self())),
+
+
+    %% then
+    ?assertReceivedMatch({on_complete, {ok, DataSize}}, ?TIMEOUT),
+    stop_counter(CounterPid),
+    ?assertReceivedMatch({counter, 1}, ?TIMEOUT).
+
+restart_rtransfer_server_test(Config) ->
+    %% test fetching data of size smaller then rtransfer block_size
+    [Worker1, Worker2 | _] = ?config(op_worker_nodes, Config),
+
+    %% given
+    DataSize = 32,
+    Data = generate_binary(DataSize),
+    ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
+    WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
+    CounterPid = spawn(?MODULE, counter, [0]),
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+
+    %% when
+    ?assertMatch({ok, _}, start_gateway(Worker1, RtransferOpts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, RtransferOpts2)),
+
+    Ref = prepare_fetch_request(Worker1, Worker2, ?TEST_FILE_UUID, ?TEST_OFFSET, DataSize),
+    ok = rpc:call(Worker2, gen_server2, stop, [{global, rtransfer}, test_reason, infinity]),
+    fetch_data(Worker1, Ref, notify_fun(CounterPid), on_complete_fun(self())),
+
+    %% then
+    ?assertReceivedMatch({on_complete, {ok, DataSize}}, ?TIMEOUT),
+    stop_counter(CounterPid),
+    ?assertReceivedMatch({counter, 1}, ?TIMEOUT).
+
 exact_block_size_fetch_test(Config) ->
     %% test fetching data of size equal to rtransfer block_size
     [Worker1, Worker2 | _] = ?config(op_worker_nodes, Config),
@@ -113,8 +174,8 @@ exact_block_size_fetch_test(Config) ->
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     CounterPid = spawn(?MODULE, counter, [0]),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -140,8 +201,8 @@ more_than_block_fetch_test(Config) ->
     CounterPid = spawn(?MODULE, counter, [0]),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, ?TEST_BLOCK_SIZE}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -173,8 +234,8 @@ more_than_block_fetch_test2(Config) ->
             {ok, ?FILE_HANDLE2, byte_size(_Data) }
         end
     },
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -201,8 +262,8 @@ cancel_fetch_test(Config) ->
         end
     },
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     Ref = prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -226,12 +287,12 @@ many_requests_test(Config) ->
     CounterPid = spawn(?MODULE, onCompleteCounter, [#{ok => 0, errors => 0, reason => []}]),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
-    ?assertMatch({ok, _}, start_rtransfer(Worker1, RtransferOpts1)),
-    ?assertMatch({ok, _}, start_rtransfer(Worker2, RtransferOpts2)),
+    ?assertMatch({ok, _}, start_gateway(Worker1, RtransferOpts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, RtransferOpts2)),
     Refs = generate_requests_to_many_files(RequestsNum, Worker1, Worker2, DataSize),
     fetch_many(Refs, Worker1, notify_fun(), on_complete_fun(CounterPid)),
 
@@ -252,12 +313,12 @@ many_same_requests_test(Config) ->
     CounterPid = spawn(?MODULE, onCompleteCounter, [#{ok => 0, errors => 0, reason => []}]),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
-    ?assertMatch({ok, _}, start_rtransfer(Worker1, RtransferOpts1)),
-    ?assertMatch({ok, _}, start_rtransfer(Worker2, RtransferOpts2)),
+    ?assertMatch({ok, _}, start_gateway(Worker1, RtransferOpts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, RtransferOpts2)),
     Refs = generate_many_same_requests(RequestsNum, Worker1, Worker2, DataSize),
     fetch_many(Refs, Worker1, notify_fun(), on_complete_fun(CounterPid)),
 
@@ -281,12 +342,12 @@ many_requests_to_one_file(Config) ->
     CounterPid = spawn(?MODULE, onCompleteCounter, [#{ok => 0, errors => 0, reason => []}]),
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
-    ?assertMatch({ok, _}, start_rtransfer(Worker1, RtransferOpts1)),
-    ?assertMatch({ok, _}, start_rtransfer(Worker2, RtransferOpts2)),
+    ?assertMatch({ok, _}, start_gateway(Worker1, RtransferOpts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, RtransferOpts2)),
     Refs = generate_requests_to_one_file(RequestsNum, Worker1, Worker2, Chunk),
     fetch_many(Refs, Worker1, notify_fun(), on_complete_fun(CounterPid)),
 
@@ -311,8 +372,8 @@ error_open_fun_test(Config) ->
 
     %% when
     prepare_rtransfer(
-        {Worker1, [[get_binding(Worker1) | RtransferOpts2]]},
-        {Worker2, [[get_binding(Worker2) | RtransferOpts2]]},
+        {Worker1, [RtransferOpts2]},
+        {Worker2, [RtransferOpts2]},
         ?TEST_FILE_UUID, ?TEST_OFFSET, DataSize,
         notify_fun(), on_complete_fun(self())
     ),
@@ -328,13 +389,13 @@ error_read_fun_test(Config) ->
     DataSize = ?TEST_BLOCK_SIZE,
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, DataSize}),
     ReadFunOpt = make_opt_fun(read_fun, {error, ?FILE_HANDLE2, test_error}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer(
-        {Worker1, [[get_binding(Worker1) | RtransferOpts1]]},
-        {Worker2, [[get_binding(Worker2) | RtransferOpts2]]},
+        {Worker1, [RtransferOpts1]},
+        {Worker2, [RtransferOpts2]},
         ?TEST_FILE_UUID, ?TEST_OFFSET, DataSize,
         notify_fun(), on_complete_fun(self())
     ),
@@ -351,13 +412,13 @@ error_write_fun_test(Config) ->
     Data = generate_binary(DataSize),
     WriteFunOpt = make_opt_fun(write_fun, {error, ?FILE_HANDLE2, test_error}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, Data}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer(
-        {Worker1, [[get_binding(Worker1) | RtransferOpts1]]},
-        {Worker2, [[get_binding(Worker2) | RtransferOpts2]]},
+        {Worker1, [RtransferOpts1]},
+        {Worker2, [RtransferOpts2]},
         ?TEST_FILE_UUID, ?TEST_OFFSET, DataSize,
         notify_fun(), on_complete_fun(self())
     ),
@@ -375,8 +436,8 @@ offset_greater_than_file_size_test(Config) ->
     Offset = DataSize + 10,
     WriteFunOpt = make_opt_fun(write_fun, {ok, ?FILE_HANDLE2, 0}),
     ReadFunOpt = make_opt_fun(read_fun, {ok, ?FILE_HANDLE2, <<"">>}),
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -416,8 +477,8 @@ request_bigger_than_file_test(Config) ->
         end
     },
 
-    RtransferOpts1 = [ReadFunOpt, WriteFunOpt, get_binding(Worker1) | ?DEFAULT_RTRANSFER_OPTS],
-    RtransferOpts2 = [ReadFunOpt, WriteFunOpt, get_binding(Worker2) | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts1 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
+    RtransferOpts2 = [ReadFunOpt, WriteFunOpt | ?DEFAULT_RTRANSFER_OPTS],
 
     %% when
     prepare_rtransfer({Worker1, RtransferOpts1}, {Worker2, RtransferOpts2},
@@ -433,28 +494,63 @@ request_bigger_than_file_test(Config) ->
 %%%===================================================================
 
 init_per_suite(Config) ->
-    [{?CTH_ENV_UP, ?DISABLE} | Config].
-
-init_per_testcase(_Case, Config) ->
-    NewConfig = test_node_starter:prepare_test_environment(Config, ?MODULE),
+    Posthook = fun(NewConfig) ->
+        Workers = ?config(op_worker_nodes, NewConfig),
+        application:start(ssl),
+        hackney:start(),
+        lists:foreach(fun(Worker) ->
+            start_applier(Worker, ?REMOTE_APPLIER)
+        end, Workers),
+        mock_providers_get_details(Workers),
+        NewConfig
+    end,
     ssl:start(),
     hackney:start(),
-    [Worker1, Worker2 | _] = ?config(op_worker_nodes, NewConfig),
-    start_applier(Worker1, ?REMOTE_APPLIER),
-    start_applier(Worker2, ?REMOTE_APPLIER),
-    NewConfig.
+    [
+        {?ENV_UP_POSTHOOK, Posthook},
+        {?LOAD_MODULES, [initializer]}
+        | Config
+    ].
+
+init_per_testcase(_Case, Config) ->
+    Workers = ?config(op_worker_nodes, Config),
+    lists:foreach(fun(Worker) ->
+        start_applier(Worker, ?REMOTE_APPLIER)
+    end, Workers),
+    Config.
 
 end_per_testcase(_Case, Config) ->
     Workers = ?config(op_worker_nodes, Config),
     lists:foreach(fun stop_applier/1, Workers),
+    Config.
+
+end_per_suite(Config) ->
+    Workers = ?config(op_worker_nodes, Config),
     hackney:stop(),
     ssl:stop(),
-    test_node_starter:clean_environment(Config).
+    unmock_providers_get_details(Workers).
 
 %%%===================================================================
 %%% Internal functions
 %%%===================================================================
 
+mock_providers_get_details(Nodes) ->
+    [Node1, Node2] = Nodes,
+    ok = test_utils:mock_new(Nodes, oz_providers),
+    ok = test_utils:mock_expect(Nodes, oz_providers, get_details, fun(Auth, ProviderId) ->
+        case ProviderId of
+            <<"p1">> ->
+                {ok, #provider_details{urls = [list_to_binary(utils:get_host(Node2))]}};
+            <<"p2">> ->
+                {ok, #provider_details{urls = [list_to_binary(utils:get_host(Node1))]}};
+            _ ->
+                meck:passthrough([Auth, ProviderId])
+        end
+    end).
+
+unmock_providers_get_details(Nodes) ->
+    ok = test_utils:mock_unload(Nodes, oz_providers).
+
 get_nodes() ->
     fun(ProviderId) ->
         [{utils:get_host(ProviderId), ?RTRANSFER_PORT}]
@@ -493,11 +589,6 @@ on_complete_fun(Pid) ->
         ok
     end.
 
-get_binding(Node) ->
-    Ip = get_node_ip(Node),
-    {ok, InetIp} = inet:parse_address(Ip),
-    {bind, [InetIp]}.
-
 change_rtransfer_opt({OptKey, OptValue}, Opts) ->
     case lists:keysearch(OptKey, 1, Opts) of
         false -> [{OptKey, OptValue} | Opts];
@@ -585,24 +676,27 @@ onCompleteCounter(#{ok := OK, errors := Errors, reason := Reason} = State) ->
     end.
 
 generate_requests_to_many_files(RequestsNum, Worker1, Worker2, DataSize) ->
+    ProviderId2 = provider_id(Worker2),
     [
         remote_apply(Worker1, rtransfer, prepare_request,
-            [Worker2, <<?TEST_FILE_UUID/binary, (list_to_binary(integer_to_list(I)))/binary>>,
+            [ProviderId2, <<?TEST_FILE_UUID/binary, (list_to_binary(integer_to_list(I)))/binary>>,
                 ?TEST_OFFSET, DataSize])
         || I <- lists:seq(1, RequestsNum)
     ].
 
 generate_many_same_requests(RequestsNum, Worker1, Worker2, DataSize) ->
+    ProviderId2 = provider_id(Worker2),
     [
         remote_apply(Worker1, rtransfer, prepare_request,
-            [Worker2, <<?TEST_FILE_UUID/binary>>, ?TEST_OFFSET, DataSize])
+            [ProviderId2, <<?TEST_FILE_UUID/binary>>, ?TEST_OFFSET, DataSize])
         || _ <- lists:seq(1, RequestsNum)
     ].
 
 generate_requests_to_one_file(RequestsNum, Worker1, Worker2, Chunk) ->
+    ProviderId2 = provider_id(Worker2),
     [
         remote_apply(Worker1, rtransfer, prepare_request,
-            [Worker2, ?TEST_FILE_UUID, Offset, Chunk])
+            [ProviderId2, ?TEST_FILE_UUID, Offset, Chunk])
         || Offset <- lists:seq(0, 2 * RequestsNum - 1 , Chunk div 2)
     ].
 
@@ -622,13 +716,14 @@ get_node_ip(Node) ->
     re:replace(os:cmd(CMD), "\\s+", "", [global, {return, list}]).
 
 
-start_rtransfer(Node, RtransferOpts) ->
-    remote_apply(Node, rtransfer, start_link, [RtransferOpts]).
+start_gateway(Node, RtransferOpts) ->
+    remote_apply(Node, fslogic_worker, restart_gateway, [RtransferOpts]).
 
 prepare_fetch_request(Node1, Node2, FileUUID, Offset, DataSize) ->
 %% Node1 is the one who fetches data from Node2
+    ProviderId2 = provider_id(Node2),
     remote_apply(
-        Node1, rtransfer, prepare_request, [Node2, FileUUID, Offset, DataSize]
+        Node1, rtransfer, prepare_request, [ProviderId2, FileUUID, Offset, DataSize]
     ).
 
 fetch_data(Node, Ref, NotifyFun, OnCompleteFun) ->
@@ -638,8 +733,12 @@ cancel_fetching(Node, Ref) ->
     remote_apply(Node, rtransfer, cancel, [Ref]).
 
 prepare_rtransfer({Worker1, Ropts1}, {Worker2, Ropts2}, FileUUID, Offset, DataSize, NotifyFun, OnCompleteFun) ->
-    ?assertMatch({ok, _}, start_rtransfer(Worker1, Ropts1)),
-    ?assertMatch({ok, _}, start_rtransfer(Worker2, Ropts2)),
+    ?assertMatch({ok, _}, start_gateway(Worker1, Ropts1)),
+    ?assertMatch({ok, _}, start_gateway(Worker2, Ropts2)),
     Ref = prepare_fetch_request(Worker1, Worker2, FileUUID, Offset, DataSize),
-    NewRef = fetch_data(Worker1, Ref, NotifyFun, OnCompleteFun),
-    NewRef.
+    fetch_data(Worker1, Ref, NotifyFun, OnCompleteFun).
+
+provider_id(Node) ->
+    Hostname = list_to_binary(utils:get_host(Node)),
+    [_, Id | _] = binary:split(Hostname, <<".">>, [global]),
+    Id.
\ No newline at end of file
